# Copyright (C) 2012  Stian Ellingsen <stian@plaimi.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from itertools import product


class TiledRenderer(object):

    """Renders a Tiled map onto a surface."""

    def __init__(self, tiledmap):
        self._tiledmap = tiledmap

    def render(self, surface, offset=(0, 0), layers=None):
        if layers is None:
            layers = xrange(0, len(self._tiledmap.tilelayers))
        m = self._tiledmap
        tw = m.tilewidth
        th = m.tileheight
        gt = m.getTileImage
        x, y = offset
        dr = surface.get_clip()
        sr = dr.move(x, y)
        rows = xrange(max(0, sr.top // th),
                      min(m.height, 1 + sr.bottom // th))
        cols = xrange(max(0, sr.left // th),
                      min(m.width, 1 + sr.right // tw))
        for layer, row, col in product(layers, rows, cols):
            tile = gt(col, row, layer)
            if tile:
                surface.blit(tile, (col * tw - x, row * th - y))
