# Copyright (C) 2012  Stian Ellingsen <stian@plaimi.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pygame

from graphics.surface import SurfaceScaler


class ScreenScaler(object):

    def __init__(self, min_size, max_size):
        self._min_size = min_size
        self._max_size = max_size
        self._scaler = SurfaceScaler()

    @property
    def surface(self):
        """The surface to be transferred to the display"""
        return self._scaler.source

    def resize(self, size, flags, crop=False):
        """Update the display size"""
        (minw, minh), (maxw, maxh) = self._min_size, self._max_size
        scale = max(1, min(size[0] // minw, size[1] // minh))
        w, h = min(maxw, size[0] // scale), min(maxh, size[1] // scale)
        if crop:
            w, h = max(minw, w), max(minh, h)
        r = pygame.Rect(0, 0, w * scale, h * scale)
        if not crop:
            r.center = (size[0] // 2, size[1] // 2)
        else:
            size = r.size
        screen = pygame.display.set_mode(size, flags)
        #print x, y, w, h, size, screen.get_size()
        self._scaler.scale = scale
        self._scaler.dest = screen if r.size == size else screen.subsurface(r)

    def update(self, rectangles=None):
        """Update the display with all or parts of the source surface"""
        rects = self._scaler.update(rectangles)
        if rectangles is None:
            pygame.display.update()
        else:
            x, y = self._scaler.dest.get_abs_offset()
            pygame.display.update([r.move(x, y) for r in rects])
