# Copyright (C) 2012  Stian Ellingsen <stian@plaimi.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys

import pygame

from pytmx import tmxloader
from graphics.screen import ScreenScaler
from graphics.map import TiledRenderer

pygame.init()

dinfo = pygame.display.Info()
screensize = (dinfo.current_w, dinfo.current_h)
windowsize = (screensize[0] - 128, screensize[1] - 128)

clock = pygame.time.Clock()

scrsize = width, height = 554, 352
black = (0, 0, 0)

ss = ScreenScaler((512, 352), (640, 448))

ss.resize(windowsize, pygame.RESIZABLE, True)

font = pygame.font.SysFont('monospace', 24)
text = font.render("Test", True, black)

screensize = (dinfo.current_w, dinfo.current_h)
windowsize = (screensize[0] - 128, screensize[1] - 128)

lasttime = pygame.time.get_ticks() // 1000
ctr = 0

ox = oy = 0
dox = doy = 0

tiledmap = tmxloader.load_pygame("data/levels/mountain_path.tmx", pixelalpha=True)
maprenderer = TiledRenderer(tiledmap)

changed = True

dt = 0

while True:
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            pygame.quit()
            sys.exit(0)
        elif event.type == pygame.KEYDOWN:
            if event.key == pygame.K_f:
                if pygame.display.get_surface().get_flags() & pygame.FULLSCREEN:
                    ss.resize(windowsize, pygame.RESIZABLE, True)
                else:
                    ss.resize(screensize, pygame.FULLSCREEN)
                changed = True
            elif event.key == pygame.K_LEFT:
                dox -= 1
            elif event.key == pygame.K_RIGHT:
                dox += 1
            elif event.key == pygame.K_UP:
                doy -= 1
            elif event.key == pygame.K_DOWN:
                doy += 1
        elif event.type == pygame.VIDEORESIZE:
            if pygame.display.get_surface().get_flags() & pygame.FULLSCREEN:
                print 'wtf'
                screensize = event.size
                ss.resize(scrsize, pygame.FULLSCREEN)
            else:
                windowsize = event.size
                ss.resize(windowsize, pygame.RESIZABLE)
            changed = True
    sscreen = ss.surface
    w, h = sscreen.get_size()
    oox, ooy = int(ox), int(oy)
    ox += dox * 0.05 * dt
    oy += doy * 0.05 * dt
    if ox < 0:
        dox, ox = 0, 0
    elif ox > tiledmap.width * tiledmap.tilewidth:
        dox, ox = 0, tiledmap.width * tiledmap.tilewidth
    if oy < 0:
        doy, oy = 0, 0
    elif oy > tiledmap.height * tiledmap.tileheight:
        doy, oy = 0, tiledmap.height * tiledmap.tileheight
    if (int(ox) != oox) or (int(oy) != ooy):
        changed = True
    if changed:
        sscreen.fill(black)
        maprenderer.render(sscreen, (int(ox) - w // 2, int(oy) - h // 2))
        p = ((sscreen.get_width() + 1 - text.get_width()) // 2, 1)
        sscreen.blit(text, p)
        ss.update()
    ctr += 1
    t = pygame.time.get_ticks() // 1000
    dt = clock.tick(0 if changed else 60)
    changed = False
    if t != lasttime:
        lasttime = t
        print(ctr)
        ctr = 0
