"""
Simple menu system.  'Menu' is the controller.  It creates hard coded 'SubMenu's
that have elements. 'Menu' does 'draw()' which calls a 'Submenu' 'draw()', which
blits all of the 'elements{}' in that 'Submenu'.  'Menu' only draws its
'active_menu'.
"""
from pygame import font


class Menu():

    def __init__(self, parent):
        # Surface to draw to.
        self.parent = parent
        # The submenu to be drawn.  0 = mainmenu, 1 = gamemenu and 2 =
        # settings.
        self.active_menu = 0

        # Load images.
        self.new = MenuElement(parent, "NEW GAME")
        self.save = MenuElement(parent, "SAVE GAME")
        self.load = MenuElement(parent, "LOAD GAME")
        self.settings = MenuElement(parent, "SETTINGS")
        self.exit = MenuElement(parent, "EXIT GAME")

        # List of submenus.
        self.menus = []
        # Create submenus.
        self.main_menu = SubMenu()
        self.game_menu = SubMenu()
        self.settings_menu = SubMenu()
        # Add them to the list.
        self.menus.append(self.main_menu)
        self.menus.append(self.game_menu)
        self.menus.append(self.settings_menu)

        # Add items to the mainmenu.
        self.menus[0].add_element(self.new, 0)
        self.menus[0].add_element(self.load, 1)
        self.menus[0].add_element(self.settings, 2)
        self.menus[0].add_element(self.exit, 3)

        # Add items to the gamemenu.
        self.menus[1].add_element(self.save, 0)
        self.menus[1].add_element(self.settings, 1)
        self.menus[1].add_element(self.exit, 2)

    def activate(self, menu):
        """Set which submenu is active."""
        self.active_menu = menu

    def draw(self):
        """Draw the active submenu."""
        self.menus[self.active_menu].draw(self.parent)

    def select(self, upP):
        """Change the active element in the active submenu."""
        self.menus[self.active_menu].select(upP)


class SubMenu():
    def __init__(self):
        # Menu elements.
        self.elements = {}
        # Selected element.
        self.selected = 0

    def add_element(self, element, position):
        """Add an element to the menu."""
        self.elements[position] = element
        if position is 0:
            element.selectedP = True
            element.update()

    def select(self, upP):
        """Change the selected element."""
        new_selected = self.selected - (1 if upP else -1)
        if new_selected < 0:
            new_selected = 0
        if new_selected >= len(self.elements):
            new_selected = len(self.elements) - 1
        if self.selected is not new_selected:
            self.elements[self.selected].selectedP = False
            self.elements[self.selected].update()
            self.selected = new_selected
            self.elements[self.selected].selectedP = True
            self.elements[self.selected].update()

    def draw(self, surface):
        """Draw all elements in the menu."""
        for i, j in self.elements.iteritems():
            j.draw(i)


class MenuElement():
    def __init__(self, parent, text="", colour=(0, 0, 0), size=32):
        self.text = text
        self.parent = parent
        self.colour = colour
        self.size = size
        self.selectedP = False
        self.rect = None
        self.parent = parent
        self.update()

    def draw(self, position):
        self.rect.top = 50 + 100 * position
        if ((self.surface) and (self.rect)):
            self.parent.blit(self.surface, self.rect)

    def update(self):
        self.size = 128 if self.selectedP else 96
        self.font = font.SysFont("None", self.size)
        self.surface = self.font.render(self.text, True, self.colour)
        if (self.rect):
            self.rect = self.surface.get_rect(center=self.rect.center)
        else:
            self.rect = self.surface.get_rect()
        self.rect.center = self.parent.get_rect().center
import pygame
pygame.init()
window_width = 640
window_height = 480
screen = pygame.display.set_mode((window_width, window_height))
running = True
menu = Menu(screen)
pygame.display.flip()
while running:
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            running = False
        if event.type == pygame.KEYDOWN:
            if event.key == pygame.K_UP:
                menu.select(True)
            if event.key == pygame.K_DOWN:
                menu.select(False)
    screen.fill((255, 255, 255))
    menu.draw()
    pygame.display.flip()
