"""
A 'Weapon' is a type of item that is used when 'Character's 'hit()' and
'damage(crit)'.

It has a 'die' that is used when calculating 'damage(crit)'.  In addition it has
a 'crit_mod'.  Upon a critical hit, the 'damage' is multiplied with the
'crit_mod'.  A critical hit occurs when the 'Character' rolls its 'die' and gets
a result above or equal to the die size minus the 'crit_range' of the 'Weapon'.

'range_' dictates how many tiles away their strike can land.  'frequency'
dictates how many strikes it may perform per second.
"""
from nei.item import Item
from nei.dice import roll_die


class Weapon(Item):

    def __init__(self, name, slot, weight, range_, frequency, die,
                 crit_range, crit_mod):
        # Set name, slot and weight
        Item.__init__(self, name, slot, weight)
        # Range of attack in number (integers) of tiles.  Melee weapons have 1.
        self.range_ = range_
        #  Frequency of attack.  Attacks per second.
        self.frequency = frequency
        # Damage die.
        self.die = die
        # Critical hit range.  If the character rolls above the max of his die
        # minus this number. it performs a critical hit.  This is always a
        # successful hit.
        self.crit_range = crit_range
        # Critical modifier.  Upon a critical hit, the damage inflicted is
        # multiplied with this modifier.
        self.crit_mod = crit_mod

    def damage(self, crit):
        # Roll the damage die.  If there was a critical strike, multiply the
        # damage to be dealt with the critical modifier
        return roll_die(self.die) * self.crit_mod if crit else 1
